#include "wiced.h"
#include "SimpleCli.h"
#include "IoTest.h"
#ifdef USE_SUSI_API_TEST
#include "./susiTest/susiGpioTest.h"
#include "./susiTest/susiI2CTest.h"
#include "./susiTest/susiSPITest.h"
#include "./susiTest/susiADCTest.h"
#else
#include "./Peripheral/GpioTest.h"
#include "./Peripheral/I2CTest.h"
#include "./Peripheral/SPITest.h"
#include "./Peripheral/ADCTest.h"
#endif
#include "./Peripheral/UartTest.h"
#include "./Peripheral/WiFiTest.h"
#include "./Peripheral/BTTest.h"
#include "../libraries/utilities/command_console/command_console.h"

#define MAC_STR_LEN                     (18)

static wiced_bool_t ota2_get_mac_addr_text(wiced_mac_t *mac, char* out, int out_len);

int IOTest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;
    
	simple_cmds cmds[] = {
			{ "I/O Function Test" /*title*/,          		0, NULL          },
			{ "I2C",										0, I2CTest_Menu  },
			{ "SPI0",										0, SPITest_Menu  },
			{ "GPIO",   									0, GpioTest_Menu },
			{ "Uart-2",     								0, UartTest_Menu },
			{ "ADC",										0, ADCTest_Menu  },
			{ "Wi-Fi",										0, WiFiTest_Menu },
			{ "Bluetooth",									0, BTTest_Menu   },

			{ NULL,											0, NULL          }
	};

	return simple_cli_list(cmds);
}

int AutoTest(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	console_printf("===================================\r\n");
	console_printf("=                                 =\r\n");
	console_printf("=          %s AutoTest     =\r\n",  AP_NAME);
	console_printf("=                                 =\r\n");
	console_printf("===================================\r\n");
	console_printf("\r\n");

	console_printf("BT    test... ");
	if(BLE_Test()==(int)WICED_SUCCESS)
		console_printf("OK.\r\n");
	else
		console_printf(CONSOLE_LIGHT_RED"NG!!!\r\n"CONSOLE_NONE);

	console_printf("Wi-Fi ");
	if(WiFi_Conn()==(int)WICED_SUCCESS){
		if(Ping_Command(NULL, 2)==ERR_CMD_OK)
			console_printf("test... OK.\r\n");
		else
			console_printf(CONSOLE_LIGHT_RED"Ping test NG.\r\n");
	}
	else{
		console_printf(" ===> "CONSOLE_LIGHT_RED"NG.\r\n"CONSOLE_NONE);
	}
	//wiced_wlan_connectivity_deinit( );

	console_printf("I2C   test... ");
	if(I2C_Auto_Test()==(int)WICED_SUCCESS)
		console_printf("OK.\r\n");
	else
		console_printf(CONSOLE_LIGHT_RED"NG!!!\r\n"CONSOLE_NONE);

	console_printf("SPI   test... ");
	if(SPI_Auto_Test()==(int)WICED_SUCCESS)
		console_printf("OK.\r\n");
	else
		console_printf(CONSOLE_LIGHT_RED"NG!!!\r\n"CONSOLE_NONE);

	console_printf("ADC0  test... ");
	if(ADC_Auto_Test(0)==(int)WICED_SUCCESS)
		console_printf("OK.\r\n");
	else
		console_printf(CONSOLE_LIGHT_RED"NG!!!\r\n"CONSOLE_NONE);

	console_printf("ADC1  test... ");
	if(ADC_Auto_Test(1)==(int)WICED_SUCCESS)
		console_printf("OK.\r\n");
	else
		console_printf(CONSOLE_LIGHT_RED"NG!!!\r\n"CONSOLE_NONE);

	console_printf("ADC2  test... ");
	if(ADC_Auto_Test(2)==(int)WICED_SUCCESS)
		console_printf("OK.\r\n");
	else
		console_printf(CONSOLE_LIGHT_RED"NG!!!\r\n"CONSOLE_NONE);

	console_printf("ADC3  test... ");
	if(ADC_Auto_Test(3)==(int)WICED_SUCCESS)
		console_printf("OK.\r\n");
	else
		console_printf(CONSOLE_LIGHT_RED"NG!!!\r\n"CONSOLE_NONE);

	console_printf("Uart2 test... ");
	if(Uart2_Auto_Test()==(int)WICED_SUCCESS)
		console_printf("OK.\r\n");
	else
		console_printf(CONSOLE_LIGHT_RED"NG!!!\r\n"CONSOLE_NONE);

	console_printf("GPIO  test... ");
	if(Gpio_Auto_Test()==(int)WICED_SUCCESS)
		console_printf("OK.\r\n");
	else
		console_printf(CONSOLE_LIGHT_RED"NG!!!\r\n"CONSOLE_NONE);

	console_printf("\r\nPress any key...\r\n");
	getchar();

	return simple_cli_list(main_menu_cmds);
}

int GetWiFiMAC(int index, char *string, int len, void *printer)
{
	if(string != NULL && len != 0) return 0;

	wiced_mac_t wiced_mac;
	char mac_str[MAC_STR_LEN] = {0};

	wwd_wifi_get_mac_address( &wiced_mac, WICED_STA_INTERFACE );
	ota2_get_mac_addr_text(&wiced_mac, mac_str, sizeof(mac_str));
	console_printf(" Wi-Fi MAC:<%s>\r\n", mac_str);
}

static wiced_bool_t ota2_get_mac_addr_text(wiced_mac_t *mac, char* out, int out_len)
{
    if((mac == NULL) || (out == NULL) || (out_len < MAC_STR_LEN))
    {
        return WICED_FALSE;
    }

    sprintf(out, "%02x:%02x:%02x:%02x:%02x:%02x",
            mac->octet[0], mac->octet[1], mac->octet[2], mac->octet[3], mac->octet[4], mac->octet[5]);

    return WICED_TRUE;
}

