#include "wiced.h"
#include "../simpleCli.h"
#include "ADCTest.h"

#define adc0	(WICED_ADC_0)
#define adc1	(WICED_ADC_1)
#define adc2	(WICED_ADC_2)
#define adc3	(WICED_ADC_3)

static int ADCRead(int index, char *string, int len, void *printer) {
	wiced_adc_t adc;
	uint16_t sample_value;

	if(string != NULL && len != 0) return 0;

    switch(index) {
        case 0:
            adc= adc0;
        break;
        
        case 1:
            adc = adc1;
        break;
        
        case 2:
            adc = adc2;
        break;
        
        case 3:
            adc = adc3;
        break;
    }
    
    wiced_adc_init( adc, 5 );
    wiced_rtos_delay_microseconds(300);
    wiced_adc_take_sample(adc, &sample_value);

	console_printf("\r\n");
	console_printf("ADC_%d Result: %d(mV)\n", index, (int)((sample_value * 3300)/4095));
	console_printf("\r\n");
	return true;
}

int ADCTest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;
    
	simple_cmds cmds[] = {
            { "ADC list" /*title*/,                 0,NULL },
            { "ADC_0",                              0,ADCRead },
            { "ADC_1",                              1,ADCRead },
            { "ADC_2",                              2,ADCRead },
            { "ADC_3",                              3,ADCRead },
            { NULL,                                 0,NULL }
    };
	return simple_cli_list(cmds);
}

int ADC_Auto_Test(int inx)
{
	wiced_result_t result;
	wiced_adc_t adc;
	uint16_t adc_in;
	uint16_t iVal;
	uint16_t iCmp;

    switch(inx)
	{
    	case 0:
    		adc=WICED_ADC_0;
    		iCmp=ADC0_VOLTAGE;
    		break;

    	case 1:
    		adc=WICED_ADC_1;
    		iCmp=ADC1_VOLTAGE;
    		break;

    	case 2:
    		adc=WICED_ADC_2;
    		iCmp=ADC2_VOLTAGE;
    		break;

    	case 3:
    		adc=WICED_ADC_3;
    		iCmp=ADC3_VOLTAGE;
    		break;

    	default:
    		console_printf("ADC index error ===>");
    		return -1;
	}

	result=wiced_adc_init(adc, 5);
    wiced_rtos_delay_microseconds(100);
    result=wiced_adc_take_sample(adc, &adc_in);
    if(result != WICED_SUCCESS){
    	console_printf("ADC_err1 ===> ");
    	wiced_adc_deinit(adc);
    	return result;
    }
    wiced_adc_deinit(adc);

    iVal=(uint16_t)((adc_in * 3300)/4095);
    if(iVal > (iCmp*1.05) || iVal < (iCmp*0.95)){
        console_printf("ADC_%d=%d(mv) ===>", index, iVal);
    	return -1;
    }

    return WICED_SUCCESS;
}


