#include "wiced.h"
#include "../simpleCli.h"
#include "I2CTest.h"

//HDC1050 Slave
#define TI_HDC1050_DEVICE_ADDR  0x40
#define NXP_PCA9545APW_DEV_ADDR  0x70


//HDC1050 Offset
#define TI_HDC1050_MANUFACTURER_ID_ADDR	0xFE
#define TI_HDC1050_MANUFACTURER_ID		0x5449

static int gBitRate = 0;
static unsigned char gSlaveAddress = TI_HDC1050_DEVICE_ADDR;
static unsigned char gOffsetAddress = TI_HDC1050_MANUFACTURER_ID_ADDR;

wiced_i2c_device_t _i2c_dev;

static int BitRate(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (%dKHz)",gBitRate == 0 ? 100 : 400);
		return 0;
	}
	console_printf("\r\n*) Please input the bitrate ( 0: 100KHz, 1: 400KHz ): \r\n");
	gBitRate = GetInt_wait();
    
    if(gBitRate < 0 || gBitRate > 1) gBitRate = 0;
	return 0;
}

static int SlaveAddress(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (0x%02X)",gSlaveAddress);
		return 0;
	}
	console_printf("\r\n*) Please input the slave address ( 0x00 ~ 0x7F ): \r\n");
	gSlaveAddress = (unsigned char)(GetInt_wait()&0xFF);
	return 0;
}


static int OffsetAddress(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) {
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (0x%02X)",gOffsetAddress);
		return 0;
	}
	console_printf("\r\n*) Please input the offset address ( 0x00 ~ 0xFF ): \r\n");
	gOffsetAddress = (unsigned char)(GetInt_wait()&0xFF);
	return 0;
}

static int Read_HDC105_ID(int index, char *string, int len, void *printer) {
	wiced_i2c_message_t msg[2];
	uint8_t reg=TI_HDC1050_MANUFACTURER_ID_ADDR;
	uint8_t val[2]={0};
	wiced_result_t result;

	if(string != NULL && len != 0)
		return 0;

    switch(gBitRate) {
        case 1:
        	_i2c_dev.speed_mode = I2C_HIGH_SPEED_MODE;
        break;
        default:
        	_i2c_dev.speed_mode = I2C_STANDARD_SPEED_MODE;
        break;
    }

    console_printf("\r\n");
    _i2c_dev.port = WICED_I2C_1;
    _i2c_dev.address = gSlaveAddress;
    _i2c_dev.address_width = I2C_ADDRESS_WIDTH_7BIT;
    _i2c_dev.flags = I2C_DEVICE_NO_DMA;
    result=wiced_i2c_init(&_i2c_dev);

    //wiced_rtos_delay_microseconds(300);

    if(result == WICED_SUCCESS)
    	result=wiced_i2c_init_tx_message(&msg[0], &reg, sizeof(reg), 3, I2C_DEVICE_NO_DMA);

    /* Initialize RX message. */
    if(result == WICED_SUCCESS)
    	result=wiced_i2c_init_rx_message(&msg[1], val, sizeof(val), 3, I2C_DEVICE_NO_DMA);


    /* Transfer. */
    if(result == WICED_SUCCESS)
        result = wiced_i2c_transfer(&_i2c_dev, msg, ARRAYSIZE(msg));
    wiced_rtos_delay_microseconds(100);

    if(result == WICED_SUCCESS){
    	console_printf("Read from 0x%02X--0x%02X\r\n",gSlaveAddress, gOffsetAddress);
    	console_printf("Result: 0x%02X%02X\r\n",val[0],val[1]);
    	console_printf("\r\n");
    }
    else{
    	//result=wiced_i2c_deinit(&_i2c_dev);
    	console_printf("Read_HDC105_ID error.\r\n");
    }

    //wiced_i2c_deinit(&_i2c_dev);
    wiced_rtos_delay_microseconds(30);
	return true;
}

int I2CTest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;
    
	simple_cmds cmds[] = {
            { "I2C Test" /*title*/,  					    0, NULL },
            { "BitRate",							    index, BitRate },
            { "Slave Address",							index, SlaveAddress },
            { "Offset Address",							index, OffsetAddress },
            { "Read TI_HDC105 ID",			        	index, Read_HDC105_ID },
			//{ "I2C-SW Channel Select",			        index, I2C_SW_Sel },
			//{ "Set I2C-SW Channel",				        index, Set_I2C_SW },
            //{ "EEP_24C32_Write",			            index, EEP_24C32_Write },
            //{ "EEP_24C32_Read",			                index, EEP_24C32_Read },
            { NULL,										0, NULL }
    };
	return simple_cli_list(cmds);
}

int I2C_Auto_Test(void)
{
	wiced_result_t result;
	wiced_i2c_message_t msg[2];
	uint8_t reg=TI_HDC1050_MANUFACTURER_ID_ADDR;
	uint8_t val[2]={0};

    _i2c_dev.port           = WICED_I2C_1;
    _i2c_dev.address        = TI_HDC1050_DEVICE_ADDR;
    _i2c_dev.address_width  = I2C_ADDRESS_WIDTH_7BIT;
    _i2c_dev.flags          = I2C_DEVICE_NO_DMA;
    _i2c_dev.speed_mode     = I2C_STANDARD_SPEED_MODE;
    result=wiced_i2c_init(&_i2c_dev);

    if(result != WICED_SUCCESS){
    	console_printf("I2C_err1 ===> ");
    	wiced_i2c_deinit(&_i2c_dev);
    	return result;
    }
    result=wiced_i2c_init_tx_message(&msg[0], &reg, sizeof(reg), 3, I2C_DEVICE_NO_DMA);
    if(result != WICED_SUCCESS){
    	console_printf("I2C_err2 ===> ");
    	wiced_i2c_deinit(&_i2c_dev);
       	return result;
    }
    /* Initialize RX message. */
   	result=wiced_i2c_init_rx_message(&msg[1], val, sizeof(val), 3, I2C_DEVICE_NO_DMA);
    if(result != WICED_SUCCESS){
    	console_printf("I2C_err3 ===> ");
    	wiced_i2c_deinit(&_i2c_dev);
       	return result;
    }
    wiced_rtos_delay_microseconds(10);
    /* Transfer. */
    result = wiced_i2c_transfer(&_i2c_dev, msg, ARRAYSIZE(msg));
    if(result != WICED_SUCCESS){
    	console_printf("I2C_err4 ===> ");
    	//wiced_i2c_deinit(&_i2c_dev);
       	return result;
    }
    //wiced_rtos_delay_microseconds(100);
    //wiced_i2c_deinit(&_i2c_dev);

    if((val[0]<<8 | val[1])==TI_HDC1050_MANUFACTURER_ID)
    	return WICED_SUCCESS;
    else{
    	console_printf("MANUFACTURER_ID=%d ===>", (val[0]<<8 | val[1]));
    	return -1;
    }
}



