#include "wiced.h"
#include "../simpleCli.h"
#ifdef USE_SUSI_API_TEST
#include "../susiTest/susiGpioTest.h"
#include "../susiTest/susiI2CTest.h"
#include "../susiTest/susiSPITest.h"
#include "../susiTest/susiADCTest.h"
#else
#include "GpioTest.h"
#include "I2CTest.h"
#include "SPITest.h"
#include "ADCTest.h"
#endif
#include "UartTest.h"
#include "WiFiTest.h"
#include "BTTest.h"
#include "internal/wiced_internal_api.h"
#include "../libraries/utilities/command_console/command_console.h"

static char DestIP[20]="192.168.1.1";
static char tempIP[20]="192.168.255.255";

int WiFi_Conn(void){
	wiced_result_t result = WICED_SUCCESS;
	wiced_config_ap_entry_t* ap;
    wiced_ip_address_t ipv4_address;

    wiced_dct_read_lock( (void**) &ap, WICED_FALSE, DCT_WIFI_CONFIG_SECTION, (uint32_t) ( OFFSETOF(platform_dct_wifi_config_t,stored_ap_list)), sizeof(wiced_config_ap_entry_t) );

    console_printf("join to *%s ", ap->details.SSID.value);

	result=wiced_network_up( WICED_STA_INTERFACE, WICED_USE_EXTERNAL_DHCP_SERVER, NULL );

	if(	result==WICED_SUCCESS){
		result=wiced_ip_up( WICED_STA_INTERFACE, WICED_USE_EXTERNAL_DHCP_SERVER, NULL );

		if(result!=WICED_SUCCESS){
			wiced_leave_ap(WICED_STA_INTERFACE);
			console_printf( ("NG-2\r\n") );
		}
		else{
			WPRINT_APP_DEBUG( ("OK\r\n") );
			wiced_ip_get_ipv4_address( WICED_STA_INTERFACE, &ipv4_address );
			console_printf("IP:%u.%u.%u.%u ", (uint8_t)(GET_IPV4_ADDRESS(ipv4_address) >> 24),
                    						 (uint8_t)(GET_IPV4_ADDRESS(ipv4_address) >> 16),
											 (uint8_t)(GET_IPV4_ADDRESS(ipv4_address) >> 8),
											 (uint8_t)(GET_IPV4_ADDRESS(ipv4_address) >> 0));

			sprintf(DestIP, "%d.%d.%d.1", (uint8_t)(GET_IPV4_ADDRESS(ipv4_address) >> 24),
					 	 	 	 	 	  (uint8_t)(GET_IPV4_ADDRESS(ipv4_address) >> 16),
										  (uint8_t)(GET_IPV4_ADDRESS(ipv4_address) >> 8));
		}

		return (int)result;
	}
	else{
		console_printf("fail ");
		return (int)result;
	}
}

int Ping_Command(char *ip, int times){
	int argc=4;
	int temp[4];
	int result;
	char buf[20];
	char *argv[]={"ping\0",
				  "192.168.1.1\0",
	              "-n\0",
	              "1\0"};

	sprintf(buf, "%d", times);
	argv[3]=buf;

	if(ip==NULL)
		argv[1]=DestIP;
	else{
		result=sscanf( ip, "%d.%d.%d.%d", &temp[0], &temp[1], &temp[2], &temp[3] );
		if(result==4)
			argv[1]=ip;
		else
			return ERR_BAD_ARG;
	}
	return ping(argc,  argv);
}

static int PingTest(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	console_printf( ("Ping IP:%s "
			"..."), DestIP );
	int result=Ping_Command(DestIP, 5);
	if(result==ERR_CMD_OK)
		console_printf( ("OK.\r\n") );
	else if(result==ERR_LAST_ERROR)
		console_printf("Ping have lose => NG.\r\n");
	else
		console_printf("NG.\r\n");
	return true;
}

static int InputIP(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0){
		snprintf(string, len > INBUF_SIZE ? INBUF_SIZE : len," (%s)",DestIP);
		return 0;
	}

	int leng;
	int result;
	int temp[4];

	console_printf("\r\n*) Please input destination IP: \r\n");
	GetStr_wait(tempIP, &leng);
	memcpy(tempIP, &tempIP[1], strlen(tempIP));		// for fixed GetStr_wait() bug


	result=sscanf( tempIP, "%d.%d.%d.%d", &temp[0], &temp[1], &temp[2], &temp[3] );

	if((result==4) && (temp[0]<256) && (temp[1]<256) && (temp[2]<256) && (temp[3]<256)){
		strcpy(DestIP, tempIP);
	}
	else{
		console_printf("\r\nIP format error, Press any key...");
		getchar();
	}

	return 0;
}

int WiFiTest_Menu(int index, char *string, int len, void *printer) {
	if(string != NULL && len != 0) return 0;

	simple_cmds cmds[] = {
            { "Wi-Fi list" /*title*/,               NULL,NULL  },
            { "Ping",                            	0,PingTest },
			{ "Destination IP",                    	0,InputIP  },
            { NULL,                                 NULL,NULL  }
    };

	simple_cmds cmds_pre[] = {
			{ "I/O Function Test" /*title*/,          		0, NULL          },
			{ "I2C",										0, I2CTest_Menu  },
			{ "SPI0",										0, SPITest_Menu  },
			{ "GPIO Test",									0, GpioTest_Menu  },
			{ "Uart-2 Test",								0, UartTest_Menu },
			{ "ADC",										0, ADCTest_Menu  },
			{ "Wi-Fi",										0, WiFiTest_Menu },
			{ "Bluetooth",									0, BTTest_Menu   },

			{ NULL,											0, NULL          }
	};


	if(	WiFi_Conn()==(int)WICED_SUCCESS)
		return simple_cli_list(cmds);
	else{
		console_printf("NG.\r\n");
		getchar();
		return simple_cli_list(cmds_pre);
	}
}

