#include "wiced.h"
#include "Susi4.h"
#include "susi_spi.h"

static wiced_spi_device_t  *spiHandle[SPI_MAX_BUS]={&wiced_spi_flash};
static bool g_bInited = false;
static void *g_callback = NULL;

void SusiSPI_Init(void)
{
	if(!g_bInited) {
		int i;
		for(i=0;i<SPI_MAX_BUS;i++)
		{
		    spiHandle[i]->speed=1000000;
            wiced_spi_init(spiHandle[i]);
		}
		g_bInited = true;
	}
}

/*****************************************************************************************************/
SusiStatus_t SUSI_API SusiSPIReadWrite(
	SusiId_t Id,		/* IN		SPI Bus Id */
	uint8_t *pWBuffer,	/* INOPT	Write Data pBuffer */
	uint8_t *pRBuffer,	/* OUTOPT	Read Data pBuffer */
	uint32_t coSize		/* IN		Number of Bytes */
    )
{
    wiced_spi_message_segment_t spi_msg;
    wiced_result_t result;

	if(Id >= SPI_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiSPI_Init();
    
    spi_msg.tx_buffer=pWBuffer;
    spi_msg.rx_buffer=pRBuffer;
    spi_msg.length=coSize;

    result=wiced_spi_transfer( spiHandle[Id], &spi_msg, 1 );
    if(result==WICED_SUCCESS)
	    return SUSI_STATUS_SUCCESS;
    else
        return result;
}

SusiStatus_t SUSI_API SusiSPIRead(
	SusiId_t Id,		/* IN	SPI Bus Id */
	uint8_t *pBuffer,	/* OUT	Transfer Data pBuffer */
	uint32_t ReadLen	/* IN	Byte Count to read */
    )
{
    wiced_spi_message_segment_t spi_msg;
    wiced_result_t result;

	if(Id >= SPI_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiSPI_Init();

    spi_msg.tx_buffer=NULL;
    spi_msg.rx_buffer=pBuffer;
    spi_msg.length=ReadLen;
    result=wiced_spi_transfer( spiHandle[Id], &spi_msg, 1 );

    if(result==WICED_SUCCESS)
	    return SUSI_STATUS_SUCCESS;
    else
        return result;
}

SusiStatus_t SUSI_API SusiSPIWrite(
	SusiId_t Id,		/* IN	SPI Bus Id */
	uint8_t *pBuffer,	/* IN	Transfer Data pBuffer */
	uint32_t ByteCnt	/* IN	Byte Count to write */
    )
{
    wiced_spi_message_segment_t spi_msg;
    wiced_result_t result;

	if(Id >= SPI_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiSPI_Init();

    spi_msg.tx_buffer=pBuffer;
    spi_msg.rx_buffer=NULL;
    spi_msg.length=ByteCnt;
    result=wiced_spi_transfer( spiHandle[Id], &spi_msg, 1 );

    if(result==WICED_SUCCESS)
	    return SUSI_STATUS_SUCCESS;
    else
        return result;
}

SusiStatus_t SUSI_API SusiSPIGetFrequency(
	SusiId_t Id,		/* IN	SPI Bus Id */
	uint32_t *pFreq		/* OUT	SPI Bus Frequency (KHz) */
	) {
	if(Id >= SPI_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiSPI_Init();

	*pFreq = (spiHandle[Id]->speed/1000);
	return SUSI_STATUS_SUCCESS;
}

SusiStatus_t SUSI_API SusiSPISetFrequency(
	SusiId_t Id,		/* IN	SPI Bus Id */
	uint32_t Freq		/* IN	SPI Bus Frequency (KHz) */
	) {
	if(Id >= SPI_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiSPI_Init();

	spiHandle[Id]->speed = (Freq*1000);
	return SUSI_STATUS_SUCCESS;
}



SusiStatus_t SUSI_API SusiSPIGetMode(
	SusiId_t Id,			/* IN	SPI Bus Id */
	uint32_t *pSize,		/* OUT	The number of bits per frame */
	uint32_t *pFormat,		/* OUT	SPI frame format */
	uint32_t *pBlocking		/* OUT	block or non-blocking(callback) */
	) {
	if(Id >= SPI_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiSPI_Init();

	pBlocking = g_callback;
	*pSize = spiHandle[Id]->bits;
	*pFormat = spiHandle[Id]->mode;
	return SUSI_STATUS_SUCCESS;
}

SusiStatus_t SUSI_API SusiSPISetMode(
	SusiId_t Id,			/* IN	SPI Bus Id */
	uint32_t size,			/* IN	The number of bits per frame */
	uint32_t format,		/* IN	SPI frame format */
	uint32_t blocking,		/* IN	block or non-blocking(callback) */
	void 	 *callback		/* IN	if non-blocking, use callback function */
	) {
	if(Id >= SPI_MAX_BUS) return SUSI_STATUS_INVALID_PARAMETER;
	SusiSPI_Init();

	//spiParams[Id].transferMode = blocking;
	UNUSED(blocking);
    UNUSED(callback);
    
	if(size==8 || size==16)
	    spiHandle[Id]->bits = size;
	spiHandle[Id]->mode = format;
	g_callback = NULL;
	return SUSI_STATUS_SUCCESS;
}

SusiStatus_t SUSI_API SusiSPIGetCaps(
	SusiId_t Id,		/* IN	SPI Bus Id */
	uint32_t ItemId,	/* IN	SPI Item Id */
	uint32_t *pValue	/* OUT	SPI Item value */
	) {
	return SUSI_STATUS_UNSUPPORTED;
}


