/*
 * Copyright 2017, Cypress Semiconductor Corporation or a subsidiary of 
 * Cypress Semiconductor Corporation. All Rights Reserved.
 * 
 * This software, associated documentation and materials ("Software"),
 * is owned by Cypress Semiconductor Corporation
 * or one of its subsidiaries ("Cypress") and is protected by and subject to
 * worldwide patent protection (United States and foreign),
 * United States copyright laws and international treaty provisions.
 * Therefore, you may use this Software only as provided in the license
 * agreement accompanying the software package from which you
 * obtained this Software ("EULA").
 * If no EULA applies, Cypress hereby grants you a personal, non-exclusive,
 * non-transferable license to copy, modify, and compile the Software
 * source code solely for use in connection with Cypress's
 * integrated circuit products. Any reproduction, modification, translation,
 * compilation, or representation of this Software except as specified
 * above is prohibited without the express written permission of Cypress.
 *
 * Disclaimer: THIS SOFTWARE IS PROVIDED AS-IS, WITH NO WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT, IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. Cypress
 * reserves the right to make changes to the Software without notice. Cypress
 * does not assume any liability arising out of the application or use of the
 * Software or any product or circuit described in the Software. Cypress does
 * not authorize its products for use in any products where a malfunction or
 * failure of the Cypress product may reasonably be expected to result in
 * significant property damage, injury or death ("High Risk Product"). By
 * including Cypress's product in a High Risk Product, the manufacturer
 * of such system or application assumes all risk of such use and in doing
 * so agrees to indemnify Cypress against all liability.
 */

/** @file
 *
 * GPIO API Application
 *
 * This application demonstrates how to use the WICED GPIO API
 * to toggle LEDs and read button states
 *
 * Features demonstrated
 *  - GPIO API
 *
 */

#include "wiced.h"

/******************************************************
 *                      Macros
 ******************************************************/

/******************************************************
 *                    Constants
 ******************************************************/

/******************************************************
 *                   Enumerations
 ******************************************************/

/******************************************************
 *                 Type Definitions
 ******************************************************/

/******************************************************
 *                    Structures
 ******************************************************/

/******************************************************
 *               Static Function Declarations
 ******************************************************/
void gpio_reconfig(void);
/******************************************************
 *               Variable Definitions
 ******************************************************/
extern platform_gpio_t platform_gpio_pins[] ;
/******************************************************
 *               Function Definitions
 ******************************************************/

void application_start( )
{
	wiced_bool_t result_led2;

    wiced_gpio_output_high( CB_PWR_ON );
    wiced_gpio_output_high( nCB_RESET_OUT);
    /* Initialise the WICED device */
    wiced_init();
    WPRINT_APP_INFO( ( "\r\n### Adv.factory-WISE_1530 ###\r\n\r\n" ) );
    wiced_rtos_delay_milliseconds( 1000 );

    factory_init();
    if(wiced_gpio_input_get(WICED_LED1)==WICED_TRUE){
    	wwd_wifi_set_iovar_value( IOVAR_STR_MPC, 0, WWD_STA_INTERFACE );
    	wiced_wlu_server_serial_start( STDIO_UART );
    }
    else{
        /* Remove buffering from all std streams */
    	setvbuf( stdin,  NULL, _IONBF, 0 );
    	setvbuf( stdout, NULL, _IONBF, 0 );
    	setvbuf( stderr, NULL, _IONBF, 0 );

    	simple_cli_loop(NULL);
    }
    while ( 1 )
    {
        wiced_rtos_delay_milliseconds( 1000 );
    }
}
/****************************************************************************************/
void factory_init()
{
	WPRINT_APP_INFO( ( "Factory test init.\r\n" ) );
	platform_gpio_init( &platform_gpio_pins[WICED_LED1], INPUT_PULL_DOWN );
	platform_gpio_init( &platform_gpio_pins[WICED_LED2], OUTPUT_PUSH_PULL );
	platform_gpio_output_low( &platform_gpio_pins[WICED_LED2] );
}

